@@ -2,12 +2,19 @@ package SQL::DB::Schema;
 use strict;
 use warnings;
 use Moo;
-use Log::Any qw/$log/;
 use Carp qw/confess/;
-use SQL::DB::Expr qw/_bval/;
+use SQL::DB::Expr;
 use Sub::Install qw/install_sub/;
+use Sub::Exporter -setup => {
+    exports => ['get_schema'],
+    groups  => {
+        all     => ['get_schema'],
+        default => [],
+    },
+};
 
-our $VERSION = '0.19_15';
+our $VERSION = '0.97_2';
+my %schema;
 
 # Ordinals for DBI->column_info() results
 use constant {
@@ -38,13 +45,11 @@ has 'name' => (
     required => 1,
 );
 
-has 'package_root' => (
-    is       => 'ro',
-    required => 1,
+has '_package_root' => (
+    is       => 'rw',
+    init_arg => undef,
 );
 
-has 'load' => ( is => 'ro', );
-
 has '_tables' => (
     is       => 'ro',
     init_arg => undef,
@@ -53,34 +58,19 @@ has '_tables' => (
 
 sub _getglob { no strict 'refs'; \*{ $_[0] } }
 
-around BUILDARGS => sub {
-    my $orig  = shift;
-    my $class = shift;
-    my %args  = @_;
-
-    ( $args{package_root} = $args{name} ) =~ tr/a-zA-Z0-9/_/cs;
-    $args{package_root} = __PACKAGE__ . '::' . $args{package_root};
-
-    return $class->$orig(%args);
-};
-
 sub BUILD {
     my $self = shift;
-
-    if ( $self->load ) {
-        require Module::Load;
-        Module::Load::load( $self->name );
-        $self->define( $self->name->definition );
-        $self->name->clear;
-        $log->debug( 'Loaded schema', $self->name );
-    }
+    ( my $name = $self->name ) =~ tr/a-zA-Z0-9/_/cs;
+    $self->name($name);
+    $self->_package_root( __PACKAGE__ . '::' . $name );
+    $schema{$name} = $self;
 }
 
 sub define {
     my $self = shift;
     my $data = shift;
 
-    my $package_root = $self->package_root;
+    my $package_root = $self->_package_root;
     my $tables       = $self->_tables;
 
     foreach my $colref (@$data) {
@@ -90,17 +80,14 @@ sub define {
 
         if ( !exists $tables->{$table} ) {
 
-            eval "package $srow; use Moo; extends 'SQL::DB::Expr'";
-            eval "package $urow; use Moo; extends 'SQL::DB::Expr'";
-
-  #            @{ *{ _getglob( $srow . '::ISA' ) }{ARRAY} } = ('SQL::DB::Expr');
-  #            @{ *{ _getglob( $urow . '::ISA' ) }{ARRAY} } = ('SQL::DB::Expr');
+            @{ *{ _getglob( $srow . '::ISA' ) }{ARRAY} } = ('SQL::DB::Expr');
+            @{ *{ _getglob( $urow . '::ISA' ) }{ARRAY} } = ('SQL::DB::Expr');
 
             install_sub(
                 {
                     code => sub {
                         my $table_expr = shift;
-                        return $table_expr . '.*';
+                        return '*';
                     },
                     into => $urow,
                     as   => '_columns',
@@ -110,8 +97,7 @@ sub define {
                 {
                     code => sub {
                         my $table_expr = shift;
-                        return SQL::DB::Expr->new(
-                            _txt => [ $table_expr->_alias . '.*' ] );
+                        return '*';
                     },
                     into => $srow,
                     as   => '_columns',
@@ -120,14 +106,7 @@ sub define {
         }
         $tables->{$table}++;
 
-        my $col = lc $colref->[COLUMN_NAME];
-
-        if ( $col eq 'new' ) {
-            confess "Column name 'new' (table/view '$table') clashes with "
-              . __PACKAGE__ . '!!!';
-        }
-
-        use bytes;
+        my $col  = $colref->[COLUMN_NAME];
         my $type = lc $colref->[TYPE_NAME];
 
         install_sub(
@@ -135,8 +114,8 @@ sub define {
                 code => sub {
                     my $table_expr = shift;
                     SQL::DB::Expr->new(
-                        _txt  => [ $table_expr->_alias . '.' . $col ],
-                        _type => $type,
+                        _txt   => $table_expr->_alias . '.' . $col,
+                        _btype => $type,
                     );
                 },
                 into => $srow,
@@ -152,14 +131,15 @@ sub define {
                     if (@_) {
                         my $val = shift;
                         return SQL::DB::Expr->new(
-                            _txt  => [ $col . ' = ', _bval( $val, $type ) ],
-                            _type => $type,
+                            _txt     => $col . ' = ?',
+                            _btype   => $type,
+                            _bvalues => [$val],
                         );
                     }
 
                     return SQL::DB::Expr->new(
-                        _txt  => [$col],
-                        _type => $type,
+                        _txt   => $col,
+                        _btype => $type,
                     );
                 },
                 into => $urow,
@@ -177,30 +157,14 @@ sub not_known {
     return grep { !exists $tables->{$_} } @_;
 }
 
-sub irow {
-    my $self = shift;
-
-    my @ret;
-    foreach my $name (@_) {
-        if ( !exists $self->_tables->{$name} ) {
-            confess "Table not defined in schema: $name";
-        }
-        push( @ret, sub { $name . '(' . join( ',', @_ ) . ')' } );
-        return $ret[0] unless (wantarray);
-    }
-    return @ret;
-}
-
 sub srow {
     my $self = shift;
 
     my @ret;
     foreach my $name (@_) {
-        if ( !exists $self->_tables->{$name} ) {
-            confess "Table not defined in schema: $name";
-        }
-        my $class = $self->package_root . '::Srow::' . $name;
-        my $srow = $class->new( _txt => [$name], _alias => $name );
+        my $class = $self->_package_root . '::Srow::' . $name;
+        my $srow = eval { $class->new( _txt => $name, _alias => $name ) };
+        confess "$@\nTable not defined in schema?: $name" if $@;
         return $srow unless (wantarray);
         push( @ret, $srow );
     }
@@ -212,17 +176,23 @@ sub urow {
 
     my @ret;
     foreach my $name (@_) {
-        if ( !exists $self->_tables->{$name} ) {
-            confess "Table not defined in schema: $name";
-        }
-        my $class = $self->package_root . '::Urow::' . $name;
-        my $urow = $class->new( _txt => [$name] );
+        my $class = $self->_package_root . '::Urow::' . $name;
+        my $urow = eval { $class->new( _txt => $name ) };
+        confess "$@\nTable not defined in schema?: $name" if $@;
         return $urow unless (wantarray);
         push( @ret, $urow );
     }
     return @ret;
 }
 
+# Class functions
+
+sub get_schema {
+    my $name = shift;
+    return $schema{$name} if ( exists $schema{$name} );
+    return;
+}
+
 1;
 
 # vim: set tabstop=4 expandtab:
